import invariant from 'invariant'
import { Observable } from 'rxjs'
import isObservable from 'fluorine-lib/lib/util/isObservable'

export default function combineStores(stores) {
  invariant(
    typeof stores === 'object',
    'combineStores: `stores` is expected to be an object containing Observables.')

  const keys = Object.keys(stores)

  function reduceStates() {
    const size = arguments.length;
    const state = {};

    for (let index = 0; index < size; index++) {
      const key = keys[index];
      state[key] = arguments[index];
    }

    return state;
  }

  return Observable.combineLatest(
    ...keys.map(key => {
      const store = stores[key]

      invariant(
        isObservable(store),
        'combineStores: `stores` is expected to contain Observables only.')

      return store
    }),
    reduceStates
  )
}
